package com.ruoyi.web.controller.api;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.ruoyi.app.domain.TbMyProfile;
import com.ruoyi.app.domain.TbProfileImg;
import com.ruoyi.app.domain.vo.AppMemberVo;
import com.ruoyi.app.service.ITbAppUserService;
import com.ruoyi.app.service.ITbMyProfileService;
import com.ruoyi.app.service.ITbProfileImgService;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.system.domain.TPersonalImg;
import com.ruoyi.system.domain.TPersonalMessage;
import com.ruoyi.system.domain.TPersonalModel;
import com.ruoyi.system.domain.TPersonalPage;
import com.ruoyi.system.service.ITPersonalImgService;
import com.ruoyi.system.service.ITPersonalMessageService;
import com.ruoyi.system.service.ITPersonalModelService;
import com.ruoyi.system.service.ITPersonalPageService;
import com.ruoyi.web.core.words.SensitiveWords;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 个人页 API
 */
@Api(value = "个人页 API", tags = "个人页 API")
@RestController
@RequestMapping("/template")
public class ApiTemplateController extends BaseController {

    @Autowired
    private ITPersonalImgService tPersonalImgService;
    @Autowired
    private ITPersonalPageService tPersonalPageService;
    @Autowired
    private ITPersonalModelService modelService;
    @Autowired
    private ITPersonalMessageService messageService;
    @Autowired
    private ITbMyProfileService iTbMyProfileService;
    @Autowired
    private ITbProfileImgService profileImgService;
    @Autowired
    private ITbAppUserService appUserService;
    /**
     * 查询个人页 图片列表
     */
    @ApiOperation("个人图片列表")
    @GetMapping("/allImg")
    public AjaxResult allImg(TPersonalImg tPersonalImg) {
        List<TPersonalImg> list = tPersonalImgService.list(new QueryWrapper<TPersonalImg>()
            .eq(StringUtils.isNotBlank(tPersonalImg.getEnable()),"enable", tPersonalImg.getEnable())
        );
        return AjaxResult.success(list);
    }
    /**
     * 获取个人页详细信息
     */
    @ApiOperation("获取个人页详细信息")
    @GetMapping(value = "/getPersonal/{id}")
    public AjaxResult getPersonal(@PathVariable("id") Long id) {
        TPersonalPage personalPage = tPersonalPageService.getById(id);
        if (personalPage != null && "1".equals(personalPage.getEnable())) {
            return AjaxResult.success();
        }
        return AjaxResult.success(personalPage);
    }

    /**
     * 查询个人页留言模板列表
     */
    @ApiOperation("查询个人页留言模板列表")
    @GetMapping("/allModel")
    public AjaxResult allModel() {
        LambdaQueryWrapper<TPersonalModel> lqw = new LambdaQueryWrapper<TPersonalModel>();
        lqw.eq(TPersonalModel::getEnable ,"0");
        List<TPersonalModel> list = modelService.list(lqw);
        return AjaxResult.success(list);
    }

    /**
     * 根据类型查询留言模板列表
     */
    @ApiOperation("根据类型查询留言模板列表")
    @GetMapping("/getModel/{type}")
    public AjaxResult getModel(@PathVariable("type") String type) {
        LambdaQueryWrapper<TPersonalModel> lqw = new LambdaQueryWrapper<TPersonalModel>();
        lqw.eq(TPersonalModel::getEnable ,"0");
        lqw.eq(TPersonalModel::getType,type);
        List<TPersonalModel> list = modelService.list(lqw);
        return AjaxResult.success(list);
    }


    /**
     * 新增个人页留言内容
     */
    @ApiOperation("新增个人页留言内容")
    @PostMapping("/addMessage")
    public AjaxResult add(@RequestBody TPersonalMessage tPersonalMessage) {
        if (tPersonalMessage.getAppUserId() == null){
            return AjaxResult.error("干系人id不能为空");
        }
        if (StringUtils.isBlank(tPersonalMessage.getName())){
            return AjaxResult.error("姓名不能为空");
        }
        if (StringUtils.isBlank(tPersonalMessage.getContents())){
            return AjaxResult.error("留言不能为空");
        }
        //TODO 敏感词处理
//        String name = SensitiveWords.replace(tPersonalMessage.getName());
//        tPersonalMessage.setName(name);
        tPersonalMessage.setCreateTime(new Date());
        return toAjax(messageService.save(tPersonalMessage) ? 1 : 0);
    }

    /**
     * 查询个人页留言内容列表
     */
    @ApiOperation("查询个人页留言内容列表")
    @GetMapping("/messageList")
    public TableDataInfo list(Integer pageNum,Integer pageSize,Long personalId) {
        if (personalId == null) {
            return new TableDataInfo();
        }

        pageNum = pageNum == null ? 1 : pageNum;
        pageSize = pageSize == null ? 10 : pageSize;
        PageHelper.startPage(pageNum, pageSize, "create_time desc");

        LambdaQueryWrapper<TPersonalMessage> lqw = new LambdaQueryWrapper<>();
        lqw.eq(TPersonalMessage::getPersonalId , personalId);
        List<TPersonalMessage> list = messageService.list(lqw);
        return getDataTable(list);
    }
    /**
     * 根据会员id查询个人页留言内容列表
     */
    @ApiOperation("根据会员id查询个人页留言内容列表")
    @GetMapping("/messageListByUserId")
    public TableDataInfo listByUserId(Integer pageNum,Integer pageSize,Long userId) {
        if (userId == null) {
            return new TableDataInfo();
        }

        pageNum = pageNum == null ? 1 : pageNum;
        pageSize = pageSize == null ? 10 : pageSize;
        PageHelper.startPage(pageNum, pageSize, "create_time desc");

        LambdaQueryWrapper<TPersonalMessage> lqw = new LambdaQueryWrapper<>();
        lqw.eq(TPersonalMessage::getAppUserId , userId);
        List<TPersonalMessage> list = messageService.list(lqw);
        return getDataTable(list);
    }

    @ApiOperation("个人详情列表")
    @GetMapping("/profileList")
    public AjaxResult list(TbMyProfile tbMyProfile) {
        Long appUserId = tbMyProfile.getAppUserId();
        if (appUserId == null) {
            return AjaxResult.error("参数为空");
        }
        AppMemberVo user = appUserService.getAppMember(appUserId.toString(),null);
        List<TbMyProfile> profileList = iTbMyProfileService.list(new LambdaQueryWrapper<TbMyProfile>()
                .eq(TbMyProfile::getAppUserId, appUserId)
        );
        if (profileList.size() > 0) {
            List<Long> ids = new ArrayList<>();
            profileList.forEach(item -> {
                ids.add(item.getId());
            });
            List<TbProfileImg> imgList = profileImgService.list(new LambdaQueryWrapper<TbProfileImg>()
                    .in(TbProfileImg::getProfileId, ids)
            );
            if (imgList.size() > 0) {
                profileList.forEach(item -> {
                    List<TbProfileImg> newList = new ArrayList<>();
                    imgList.forEach(img -> {
                        if (item.getId().equals(img.getProfileId())) {
                            newList.add(img);
                        }
                    });
                    item.setImgList(newList);
                });
            }
        }
        AjaxResult result = AjaxResult.success();
        result.put("user", user);
        result.put("profileList", profileList);
        return result;
    }
}
