package com.ruoyi.web.controller.api;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

import java.util.Date;
import java.util.List;
import java.util.Arrays;

import com.ruoyi.app.domain.TbMemberMiddle;
import com.ruoyi.app.domain.vo.AppFamilyVo;
import com.ruoyi.app.service.ITbMemberMiddleService;
import com.ruoyi.common.utils.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.app.domain.TbFamily;
import com.ruoyi.app.service.ITbFamilyService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 家族Controller
 * 
 * @author Alex
 * @date 2020-09-28
 */
@Api(tags = "家族管理", value = "家族管理")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/system/family" )
public class TbFamilyController extends BaseController {

    private final ITbFamilyService iTbFamilyService;
    private final ITbMemberMiddleService middleService;

    /**
     * 查询家族列表
     */
    @ApiOperation("查询家族列表")
    @PreAuthorize("@ss.hasPermi('system:family:list')")
    @GetMapping("/list")
    public TableDataInfo list(TbFamily tbFamily)
    {
        startPage();
        LambdaQueryWrapper<TbFamily> lqw = new LambdaQueryWrapper<>();
        if (StringUtils.isNotBlank(tbFamily.getCode())){
            lqw.eq(TbFamily::getCode ,tbFamily.getCode());
        }
        if (StringUtils.isNotBlank(tbFamily.getName())){
            lqw.like(TbFamily::getName ,tbFamily.getName());
        }
        if (StringUtils.isNotBlank(tbFamily.getFullName())){
            lqw.like(TbFamily::getFullName ,tbFamily.getFullName());
        }
        if (StringUtils.isNotBlank(tbFamily.getHometown())){
            lqw.eq(TbFamily::getHometown ,tbFamily.getHometown());
        }
        if (StringUtils.isNotBlank(tbFamily.getAddress())){
            lqw.eq(TbFamily::getAddress ,tbFamily.getAddress());
        }
        if (StringUtils.isNotBlank(tbFamily.getCity())){
            lqw.eq(TbFamily::getCity ,tbFamily.getCity());
        }
        List<AppFamilyVo> list = iTbFamilyService.pageList(lqw);
        return getDataTable(list);
    }


    /**
     * 根据会员id查询家族列表
     */
    @ApiOperation("根据会员id查询家族列表")
    @PreAuthorize("@ss.hasPermi('system:family:list')")
    @GetMapping("/listByUid")
    public TableDataInfo listByUid(AppFamilyVo familyVo) {
        startPage();
        List<TbFamily> list = iTbFamilyService.selectByUserId(familyVo.getUserId());
        return getDataTable(list);
    }

    /**
     * 导出家族列表
     */
    @ApiOperation("导出家族列表")
    @PreAuthorize("@ss.hasPermi('system:family:export')" )
    @Log(title = "家族" , businessType = BusinessType.EXPORT)
    @GetMapping("/export" )
    public AjaxResult export(TbFamily tbFamily) {
        LambdaQueryWrapper<TbFamily> lqw = new LambdaQueryWrapper<>(tbFamily);
        List<TbFamily> list = iTbFamilyService.list(lqw);
        ExcelUtil<TbFamily> util = new ExcelUtil<TbFamily>(TbFamily. class);
        return util.exportExcel(list, "family" );
    }

    /**
     * 获取家族详细信息
     */
    @ApiOperation("获取家族详细信息")
    @PreAuthorize("@ss.hasPermi('system:family:query')" )
    @GetMapping(value = "/{id}" )
    public AjaxResult getInfo(@PathVariable("id" ) Long id) {
        TbFamily family = iTbFamilyService.getById(id);
        return AjaxResult.success(family);
    }

    /**
     * 新增家族
     */
    @ApiOperation("新增家族")
    @PreAuthorize("@ss.hasPermi('system:family:add')" )
    @Log(title = "家族" , businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TbFamily tbFamily) {
        Date date = new Date();
        tbFamily.setCreateTime(date);
        tbFamily.setUpdateTime(date);
        return toAjax(iTbFamilyService.saveFamily(tbFamily) ? 1 : 0);
    }

    /**
     * 修改家族
     */
    @ApiOperation("修改家族")
    @PreAuthorize("@ss.hasPermi('system:family:edit')" )
    @Log(title = "家族" , businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TbFamily tbFamily) {
        return toAjax(iTbFamilyService.updateById(tbFamily) ? 1 : 0);
    }

    /**
     * 删除家族
     */
    @ApiOperation("删除家族")
    @PreAuthorize("@ss.hasPermi('system:family:remove')" )
    @Log(title = "家族" , businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}" )
    public AjaxResult remove(@PathVariable Long[] ids) {
        List<TbMemberMiddle> middles = middleService.list(new LambdaQueryWrapper<TbMemberMiddle>()
                .in(TbMemberMiddle::getFamilyId, Arrays.asList(ids))
                .eq(TbMemberMiddle::getStatus, "2")
        );
        if (middles.size() > 0) {
            return AjaxResult.error("家族中还有成员，不允许删除");
        }
        return toAjax(iTbFamilyService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
