package com.ruoyi.web.controller.api;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

import java.util.Date;
import java.util.List;
import java.util.Arrays;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import com.ruoyi.common.utils.StringUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.app.domain.TbFamilyEvents;
import com.ruoyi.app.service.ITbFamilyEventsService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 家族大事件
 * 
 * @author liuhj
 * @date 2020-10-16
 */
@Api(value = "家族大事件", tags = "家族大事件")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/system/events" )
public class TbFamilyEventsController extends BaseController {

    private final ITbFamilyEventsService iTbFamilyEventsService;

    /**
     * 查询家族大事件列表
     */
    @ApiOperation("查询家族大事件列表")
    @PreAuthorize("@ss.hasPermi('system:events:list')")
    @GetMapping("/list")
    public TableDataInfo list(TbFamilyEvents tbFamilyEvents)
    {
        startPage();
        List<TbFamilyEvents> list = iTbFamilyEventsService.queryList(tbFamilyEvents);
        return getDataTable(list);
    }

    /**
     * 导出家族大事件列表
     */
    @ApiOperation("导出家族大事件列表")
    @PreAuthorize("@ss.hasPermi('system:events:export')" )
    @Log(title = "家族大事件" , businessType = BusinessType.EXPORT)
    @GetMapping("/export" )
    public AjaxResult export(TbFamilyEvents tbFamilyEvents) {
        LambdaQueryWrapper<TbFamilyEvents> lqw = new LambdaQueryWrapper<TbFamilyEvents>(tbFamilyEvents);
        List<TbFamilyEvents> list = iTbFamilyEventsService.list(lqw);
        ExcelUtil<TbFamilyEvents> util = new ExcelUtil<TbFamilyEvents>(TbFamilyEvents. class);
        return util.exportExcel(list, "events" );
    }

    /**
     * 获取家族大事件详细信息
     */
    @ApiOperation("获取家族大事件详细信息")
    @PreAuthorize("@ss.hasPermi('system:events:query')" )
    @GetMapping(value = "/{id}" )
    public AjaxResult getInfo(@PathVariable("id" ) Long id) {
        return AjaxResult.success(iTbFamilyEventsService.getFamilyEvents(id));
    }

    /**
     * 新增家族大事件
     */
    @ApiOperation("新增家族大事件")
    @PreAuthorize("@ss.hasPermi('system:events:add')" )
    @Log(title = "家族大事件" , businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TbFamilyEvents tbFamilyEvents) {
        return toAjax(iTbFamilyEventsService.save(tbFamilyEvents) ? 1 : 0);
    }

    /**
     * 修改家族大事件
     */
    @ApiOperation("修改家族大事件")
    @PreAuthorize("@ss.hasPermi('system:events:edit')" )
    @Log(title = "家族大事件" , businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TbFamilyEvents tbFamilyEvents) {
        Date date = new Date();
        tbFamilyEvents.setUpdateTime(date);
        return toAjax(iTbFamilyEventsService.updateById(tbFamilyEvents) ? 1 : 0);
    }

    /**
     * 删除家族大事件
     */
    @ApiOperation("删除家族大事件")
    @PreAuthorize("@ss.hasPermi('system:events:remove')" )
    @Log(title = "家族大事件" , businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}" )
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(iTbFamilyEventsService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
