package com.ruoyi.web.controller.api;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

import java.util.List;
import java.util.Arrays;

import com.ruoyi.app.domain.TbAppUser;
import com.ruoyi.app.domain.TbMemberMiddle;
import com.ruoyi.app.domain.vo.AppMemberVo;
import com.ruoyi.app.service.ITbAppUserService;
import com.ruoyi.app.service.ITbMemberMiddleService;
import com.ruoyi.common.utils.bean.BeanUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.app.domain.TbFamilyMember;
import com.ruoyi.app.service.ITbFamilyMemberService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 家族成员
 * 
 * @author Administrator
 * @date 2020-10-08
 */
@Api(value = "家族成员", tags = "家族成员")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/system/member" )
public class TbFamilyMemberController extends BaseController {

    private final ITbFamilyMemberService iTbFamilyMemberService;
    private final ITbAppUserService userService;
    private final ITbMemberMiddleService middleService;

    /**
     * 查询家族成员列表
     */
    @ApiOperation("查询家族成员分页列表")
    @PreAuthorize("@ss.hasPermi('system:member:list')")
    @GetMapping("/list")
    public TableDataInfo list(AppMemberVo memberVo) {
        startPage();
        List<AppMemberVo> list = iTbFamilyMemberService.selectFamilyMember(memberVo);
        return getDataTable(list);
    }

    /**
     * 查询成员列表
     */
    @ApiOperation("查询家族成员列表")
    @PreAuthorize("@ss.hasPermi('system:member:list')")
    @GetMapping("/memberList")
    public AjaxResult memberList(AppMemberVo memberVo) {
        List<AppMemberVo> list = iTbFamilyMemberService.selectFamilyMember(memberVo);
        return AjaxResult.success(list);
    }
    /**
     * 查询未在家族树的成员列表
     */
    @ApiOperation("查询未在家族树的成员列表")
    @PreAuthorize("@ss.hasPermi('system:member:list')")
    @GetMapping("/notTreeList")
    public AjaxResult notTreeList(AppMemberVo memberVo) {
        List<AppMemberVo> list = iTbFamilyMemberService.selectMemberNotTree(memberVo);
        return AjaxResult.success(list);
    }
    /**
     * 查询已在家族树的成员列表
     */
    @ApiOperation("查询已在家族树的成员列表")
    @PreAuthorize("@ss.hasPermi('system:member:list')")
    @GetMapping("/inTreeList")
    public AjaxResult inTreeList(AppMemberVo memberVo) {
        List<AppMemberVo> list = iTbFamilyMemberService.selectMemberInTree(memberVo);
        return AjaxResult.success(list);
    }

    @ApiOperation("查询家族树 列表")
    @PreAuthorize("@ss.hasPermi('system:member:list')")
    @GetMapping("/treeList")
    public TableDataInfo selectFamilyTree(AppMemberVo memberVo){
        startPage();
        List<AppMemberVo> list = iTbFamilyMemberService.selectFamilyTree(memberVo);
        return getDataTable(list);
    }

    /**
     * 获取家族成员详细信息
     */
    @ApiOperation("获取家族成员详细信息")
    @PreAuthorize("@ss.hasPermi('system:member:query')" )
    @GetMapping(value = "/get/{familyId}/{id}" )
    public AjaxResult getInfo(@PathVariable("familyId" ) String familyId,@PathVariable("id" ) String id) {
        AppMemberVo memberVo = iTbFamilyMemberService.getMemberDetail(Long.parseLong(familyId),Long.parseLong(id));
        return AjaxResult.success(memberVo);
    }

    /**
     * 新增家族成员
     */
    @ApiOperation("新增家族成员")
    @PreAuthorize("@ss.hasPermi('system:member:add')" )
    @Log(title = "家族成员" , businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody AppMemberVo memberVo) {
        if (StringUtils.isBlank(memberVo.getMobile())){
            return AjaxResult.error("手机号不能为空");
        }
        // 后台添加，状态直接为通过 2
        memberVo.setStatus("2");
        return  iTbFamilyMemberService.add(memberVo);
    }

    /**
     * 修改家族成员
     */
    @ApiOperation("修改家族成员")
    @PreAuthorize("@ss.hasPermi('system:member:edit')" )
    @Log(title = "家族成员" , businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody AppMemberVo memberVo) {
        return iTbFamilyMemberService.edit(memberVo);
    }

    /**
     * 删除家族成员
     */
    @ApiOperation("删除家族成员")
    @PreAuthorize("@ss.hasPermi('system:member:remove')" )
    @Log(title = "家族成员" , businessType = BusinessType.DELETE)
    @DeleteMapping("/{familyId}/{ids}" )
    public AjaxResult remove(@PathVariable String familyId, @PathVariable Long[] ids) {
        if (familyId == null) {
            return AjaxResult.error("家族id不能为空");
        }
        if (ids.length <= 0) {
            return AjaxResult.error("未选择要删的成员");
        }
        return iTbFamilyMemberService.delMember(familyId,ids);
    }


    @ApiOperation("添加/修改成员-家族树")
    @PreAuthorize("@ss.hasPermi('system:member:add')" )
    @PostMapping("/saveToTree")
    public AjaxResult saveToTree(@RequestBody TbMemberMiddle member){
        if (member == null) {
            return AjaxResult.error("数据为空");
        }
        if (member.getFamilyId()==null) {
            return AjaxResult.error("家族id不能为空");
        }
        if (member.getParentId()==null) {
            return AjaxResult.error("父辈不能为空");
        }
        if (member.getMemberId()==null) {
            return AjaxResult.error("姓名不能为空");
        }
        if (member.getMemberId().equals(member.getParentId())) {
            return AjaxResult.error("不能设置自己为父辈");
        }
        if (member.getParentId().equals(member.getSpouseUserId())) {
            return AjaxResult.error("父辈与配偶不能为同一人");
        }

        if(!iTbFamilyMemberService.saveFamilyTree(member)){
            return AjaxResult.error("添加失败");
        }

        return AjaxResult.success("添加成功");
    }


    @ApiOperation(value = "逻辑删除家族树的成员", notes = "逻辑删除家族树的成员（去掉parentId），相当于这一支人将从家族树中删除")
    @PreAuthorize("@ss.hasPermi('system:member:remove')" )
    @DeleteMapping("/delToTree/{familyId}/{memberId}")
    public AjaxResult delToTree(@PathVariable Long memberId, @PathVariable Long familyId) {
        if (memberId == null || familyId ==null) {
            return AjaxResult.error("参数不能为空");
        }
        return iTbFamilyMemberService.delToTree(memberId,familyId);
    }
}
