package com.ruoyi.web.controller.api;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

import java.util.List;
import java.util.Arrays;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import com.ruoyi.common.utils.StringUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.app.domain.TbGardenImg;
import com.ruoyi.app.service.ITbGardenImgService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 家族园地图片
 * 
 * @author liuhj
 * @date 2020-10-22
 */
@Api(value = "家族园地图片", tags = "家族园地图片")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/system/gardenImg" )
public class TbGardenImgController extends BaseController {

    private final ITbGardenImgService iTbGardenImgService;

    /**
     * 查询家族园地图片列表
     */
    @ApiOperation("查询家族园地图片列表")
    @PreAuthorize("@ss.hasPermi('system:img:list')")
    @GetMapping("/list")
    public TableDataInfo list(TbGardenImg tbGardenImg)
    {
        startPage();
        LambdaQueryWrapper<TbGardenImg> lqw = new LambdaQueryWrapper<TbGardenImg>();
        if (tbGardenImg.getGardenId() != null){
            lqw.eq(TbGardenImg::getGardenId ,tbGardenImg.getGardenId());
        }
        if (StringUtils.isNotBlank(tbGardenImg.getUrl())){
            lqw.eq(TbGardenImg::getUrl ,tbGardenImg.getUrl());
        }
        List<TbGardenImg> list = iTbGardenImgService.list(lqw);
        return getDataTable(list);
    }

    /**
     * 导出家族园地图片列表
     */
    @ApiOperation("导出家族园地图片列表")
    @PreAuthorize("@ss.hasPermi('system:img:export')" )
    @Log(title = "家族园地图片" , businessType = BusinessType.EXPORT)
    @GetMapping("/export" )
    public AjaxResult export(TbGardenImg tbGardenImg) {
        LambdaQueryWrapper<TbGardenImg> lqw = new LambdaQueryWrapper<TbGardenImg>(tbGardenImg);
        List<TbGardenImg> list = iTbGardenImgService.list(lqw);
        ExcelUtil<TbGardenImg> util = new ExcelUtil<TbGardenImg>(TbGardenImg. class);
        return util.exportExcel(list, "img" );
    }

    /**
     * 获取家族园地图片详细信息
     */
    @ApiOperation("获取家族园地图片详细信息")
    @PreAuthorize("@ss.hasPermi('system:img:query')" )
    @GetMapping(value = "/{id}" )
    public AjaxResult getInfo(@PathVariable("id" ) Long id) {
        return AjaxResult.success(iTbGardenImgService.getById(id));
    }

    /**
     * 新增家族园地图片
     */
    @ApiOperation("新增家族园地图片")
    @PreAuthorize("@ss.hasPermi('system:img:add')" )
    @Log(title = "家族园地图片" , businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TbGardenImg tbGardenImg) {
        return toAjax(iTbGardenImgService.save(tbGardenImg) ? 1 : 0);
    }

    /**
     * 修改家族园地图片
     */
    @ApiOperation("修改家族园地图片")
    @PreAuthorize("@ss.hasPermi('system:img:edit')" )
    @Log(title = "家族园地图片" , businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TbGardenImg tbGardenImg) {
        return toAjax(iTbGardenImgService.updateById(tbGardenImg) ? 1 : 0);
    }

    /**
     * 删除家族园地图片
     */
    @ApiOperation("删除家族园地图片")
    @PreAuthorize("@ss.hasPermi('system:img:remove')" )
    @Log(title = "家族园地图片" , businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}" )
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(iTbGardenImgService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
