package com.ruoyi.web.controller.api;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

import java.util.List;
import java.util.Arrays;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import com.ruoyi.common.utils.StringUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.app.domain.TbMyFriends;
import com.ruoyi.app.service.ITbMyFriendsService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 我的好友
 * 
 * @author Alex
 * @date 2020-10-05
 */
@Api(value = "我的好友", tags = "我的好友")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/system/friends" )
public class TbMyFriendsController extends BaseController {

    private final ITbMyFriendsService iTbMyFriendsService;

    /**
     * 查询我的好友列表
     */
    @ApiOperation("查询我的好友列表")
    @PreAuthorize("@ss.hasPermi('system:friends:list')")
    @GetMapping("/list")
    public TableDataInfo list(TbMyFriends tbMyFriends)
    {
        List<TbMyFriends> list = iTbMyFriendsService.pageList(tbMyFriends);
        return getDataTable(list);
    }

    /**
     * 导出我的好友列表
     */
    @ApiOperation("导出我的好友列表")
    @PreAuthorize("@ss.hasPermi('system:friends:export')" )
    @Log(title = "我的好友" , businessType = BusinessType.EXPORT)
    @GetMapping("/export" )
    public AjaxResult export(TbMyFriends tbMyFriends) {
        LambdaQueryWrapper<TbMyFriends> lqw = new LambdaQueryWrapper<TbMyFriends>(tbMyFriends);
        List<TbMyFriends> list = iTbMyFriendsService.list(lqw);
        ExcelUtil<TbMyFriends> util = new ExcelUtil<TbMyFriends>(TbMyFriends. class);
        return util.exportExcel(list, "friends" );
    }

    /**
     * 获取我的好友详细信息
     */
    @ApiOperation("获取我的好友详细信息")
    @PreAuthorize("@ss.hasPermi('system:friends:query')" )
    @GetMapping(value = "/{id}" )
    public AjaxResult getInfo(@PathVariable("id" ) Long id) {
        return AjaxResult.success(iTbMyFriendsService.getById(id));
    }

    /**
     * 新增我的好友
     */
    @ApiOperation("新增我的好友")
    @PreAuthorize("@ss.hasPermi('system:friends:add')" )
    @Log(title = "我的好友" , businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TbMyFriends tbMyFriends) {
        return toAjax(iTbMyFriendsService.save(tbMyFriends) ? 1 : 0);
    }

    /**
     * 修改我的好友
     */
    @ApiOperation("修改我的好友")
    @PreAuthorize("@ss.hasPermi('system:friends:edit')" )
    @Log(title = "我的好友" , businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TbMyFriends tbMyFriends) {
        return toAjax(iTbMyFriendsService.updateById(tbMyFriends) ? 1 : 0);
    }

    /**
     * 删除我的好友
     */
    @ApiOperation("删除我的好友")
    @PreAuthorize("@ss.hasPermi('system:friends:remove')" )
    @Log(title = "我的好友" , businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}" )
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(iTbMyFriendsService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
