package com.ruoyi.web.controller.api;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

import java.util.Date;
import java.util.List;
import java.util.Arrays;

import com.ruoyi.app.domain.vo.AppMemberVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import com.ruoyi.common.utils.StringUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.app.domain.TbMyProfile;
import com.ruoyi.app.service.ITbMyProfileService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 个人详情
 * 
 * @author Alex
 * @date 2020-11-06
 */
@Api(value = "个人详情", tags = "个人详情")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/system/profile" )
public class TbMyProfileController extends BaseController {

    private final ITbMyProfileService iTbMyProfileService;

    /**
     * 查询个人详情列表
     */
    @ApiOperation("查询个人详情列表")
    @PreAuthorize("@ss.hasPermi('system:profile:list')")
    @GetMapping("/list")
    public TableDataInfo list(TbMyProfile tbMyProfile)
    {
        startPage();
        LambdaQueryWrapper<TbMyProfile> lqw = new LambdaQueryWrapper<TbMyProfile>();
        if (tbMyProfile.getAppUserId() != null){
            lqw.eq(TbMyProfile::getAppUserId ,tbMyProfile.getAppUserId());
        }
        if (StringUtils.isNotBlank(tbMyProfile.getTitle())){
            lqw.eq(TbMyProfile::getTitle ,tbMyProfile.getTitle());
        }
        if (StringUtils.isNotBlank(tbMyProfile.getCotents())){
            lqw.eq(TbMyProfile::getCotents ,tbMyProfile.getCotents());
        }
        List<TbMyProfile> list = iTbMyProfileService.list(lqw);
        return getDataTable(list);
    }

    /**
     * 导出个人详情列表
     */
    @ApiOperation("导出个人详情列表")
    @PreAuthorize("@ss.hasPermi('system:profile:export')" )
    @Log(title = "个人详情" , businessType = BusinessType.EXPORT)
    @GetMapping("/export" )
    public AjaxResult export(TbMyProfile tbMyProfile) {
        LambdaQueryWrapper<TbMyProfile> lqw = new LambdaQueryWrapper<TbMyProfile>(tbMyProfile);
        List<TbMyProfile> list = iTbMyProfileService.list(lqw);
        ExcelUtil<TbMyProfile> util = new ExcelUtil<TbMyProfile>(TbMyProfile. class);
        return util.exportExcel(list, "profile" );
    }

    /**
     * 获取个人详情详细信息
     */
    @ApiOperation("获取个人详情详细信息")
    @PreAuthorize("@ss.hasPermi('system:profile:query')" )
    @GetMapping(value = "/{id}" )
    public AjaxResult getInfo(@PathVariable("id" ) Long id) {
        return AjaxResult.success(iTbMyProfileService.getById(id));
    }

    /**
     * 新增个人详情
     */
    @ApiOperation("新增个人详情")
    @PreAuthorize("@ss.hasPermi('system:profile:add')" )
    @Log(title = "个人详情" , businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TbMyProfile tbMyProfile) {
        tbMyProfile.setCreateTime(new Date());
        return toAjax(iTbMyProfileService.save(tbMyProfile) ? 1 : 0);
    }

    /**
     * 修改个人详情
     */
    @ApiOperation("修改个人详情")
    @PreAuthorize("@ss.hasPermi('system:profile:edit')" )
    @Log(title = "个人详情" , businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TbMyProfile tbMyProfile) {
        tbMyProfile.setUpdateTime(new Date());
        return toAjax(iTbMyProfileService.updateById(tbMyProfile) ? 1 : 0);
    }

    /**
     * 删除个人详情
     */
    @ApiOperation("删除个人详情")
    @PreAuthorize("@ss.hasPermi('system:profile:remove')" )
    @Log(title = "个人详情" , businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}" )
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(iTbMyProfileService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
