package com.ruoyi.web.controller.api;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

import java.util.List;
import java.util.Arrays;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import com.ruoyi.common.utils.StringUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.app.domain.TbPublishImg;
import com.ruoyi.app.service.ITbPublishImgService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 我的状况图片
 * 
 * @author Administrator
 * @date 2020-10-02
 */
@Api(value = "我的状况图片", tags = "我的状况图片")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/system/publishImg" )
public class TbPublishImgController extends BaseController {

    private final ITbPublishImgService iTbPublishImgService;

    /**
     * 查询我的状况图片列表
     */
    @ApiOperation("查询我的状况图片列表")
    @PreAuthorize("@ss.hasPermi('system:publish:list')")
    @GetMapping("/list")
    public TableDataInfo list(TbPublishImg tbPublishImg)
    {
        startPage();
        LambdaQueryWrapper<TbPublishImg> lqw = new LambdaQueryWrapper<TbPublishImg>();
        if (tbPublishImg.getPublishId() != null){
            lqw.eq(TbPublishImg::getPublishId ,tbPublishImg.getPublishId());
        }
        if (StringUtils.isNotBlank(tbPublishImg.getUrl())){
            lqw.eq(TbPublishImg::getUrl ,tbPublishImg.getUrl());
        }
        List<TbPublishImg> list = iTbPublishImgService.list(lqw);
        return getDataTable(list);
    }

    /**
     * 导出我的状况图片列表
     */
    @ApiOperation("导出我的状况图片列表")
    @PreAuthorize("@ss.hasPermi('system:publish:export')" )
    @Log(title = "我的状况图片" , businessType = BusinessType.EXPORT)
    @GetMapping("/export" )
    public AjaxResult export(TbPublishImg tbPublishImg) {
        LambdaQueryWrapper<TbPublishImg> lqw = new LambdaQueryWrapper<TbPublishImg>(tbPublishImg);
        List<TbPublishImg> list = iTbPublishImgService.list(lqw);
        ExcelUtil<TbPublishImg> util = new ExcelUtil<TbPublishImg>(TbPublishImg. class);
        return util.exportExcel(list, "img" );
    }

    /**
     * 获取我的状况图片详细信息
     */
    @ApiOperation("获取我的状况图片详细信息")
    @PreAuthorize("@ss.hasPermi('system:publish:query')" )
    @GetMapping(value = "/{id}" )
    public AjaxResult getInfo(@PathVariable("id" ) Long id) {
        return AjaxResult.success(iTbPublishImgService.getById(id));
    }

    /**
     * 新增我的状况图片
     */
    @ApiOperation("新增我的状况图片")
    @PreAuthorize("@ss.hasPermi('system:publish:add')" )
    @Log(title = "我的状况图片" , businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TbPublishImg tbPublishImg) {
        return toAjax(iTbPublishImgService.save(tbPublishImg) ? 1 : 0);
    }

    /**
     * 修改我的状况图片
     */
    @ApiOperation("修改我的状况图片")
    @PreAuthorize("@ss.hasPermi('system:publish:edit')" )
    @Log(title = "我的状况图片" , businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TbPublishImg tbPublishImg) {
        return toAjax(iTbPublishImgService.updateById(tbPublishImg) ? 1 : 0);
    }

    /**
     * 删除我的状况图片
     */
    @ApiOperation("删除我的状况图片")
    @PreAuthorize("@ss.hasPermi('system:publish:remove')" )
    @Log(title = "我的状况图片" , businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}" )
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(iTbPublishImgService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
