package com.ruoyi.web.controller.api.family;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

import java.util.List;
import java.util.Arrays;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.app.family.album.domain.TbFamilyAlbum;
import com.ruoyi.app.family.album.service.ITbFamilyAlbumService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 家族相簿
 * 
 * @author luobo
 * @date 2020-10-12
 */
@Api(value = "家族相簿", tags = "家族相簿")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/system/family/album" )
public class TbFamilyAlbumController extends BaseController {

    private final ITbFamilyAlbumService iTbFamilyAlbumService;

    /**
     * 查询家族相簿列表
     */
    @ApiOperation("查询家族相簿列表")
    @PreAuthorize("@ss.hasPermi('system:album:list')")
    @GetMapping("/list")
    public TableDataInfo list(TbFamilyAlbum tbFamilyAlbum)
    {
        startPage();
        List<TbFamilyAlbum> list = iTbFamilyAlbumService.queryList(tbFamilyAlbum);
        return getDataTable(list);
    }

    /**
     * 导出家族相簿列表
     */
    @ApiOperation("导出家族相簿列表")
    @PreAuthorize("@ss.hasPermi('system:album:export')" )
    @Log(title = "家族相簿" , businessType = BusinessType.EXPORT)
    @GetMapping("/export" )
    public AjaxResult export(TbFamilyAlbum tbFamilyAlbum) {
        LambdaQueryWrapper<TbFamilyAlbum> lqw = new LambdaQueryWrapper<TbFamilyAlbum>(tbFamilyAlbum);
        List<TbFamilyAlbum> list = iTbFamilyAlbumService.list(lqw);
        ExcelUtil<TbFamilyAlbum> util = new ExcelUtil<TbFamilyAlbum>(TbFamilyAlbum. class);
        return util.exportExcel(list, "album" );
    }

    /**
     * 获取家族相簿详细信息
     */
    @ApiOperation("获取家族相簿详细信息")
    @PreAuthorize("@ss.hasPermi('system:album:query')" )
    @GetMapping(value = "/{id}" )
    public AjaxResult getInfo(@PathVariable("id" ) Long id) {
        return AjaxResult.success(iTbFamilyAlbumService.getEntityById(id));
    }

    /**
     * 新增家族相簿
     */
    @ApiOperation("新增家族相簿")
    @PreAuthorize("@ss.hasPermi('system:album:add')" )
    @Log(title = "家族相簿" , businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TbFamilyAlbum tbFamilyAlbum) {
        return toAjax(iTbFamilyAlbumService.save(tbFamilyAlbum) ? 1 : 0);
    }

    /**
     * 修改家族相簿
     */
    @ApiOperation("修改家族相簿")
    @PreAuthorize("@ss.hasPermi('system:album:edit')" )
    @Log(title = "家族相簿" , businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TbFamilyAlbum tbFamilyAlbum) {
        return toAjax(iTbFamilyAlbumService.updateById(tbFamilyAlbum) ? 1 : 0);
    }

    /**
     * 删除家族相簿
     */
    @ApiOperation("删除家族相簿")
    @PreAuthorize("@ss.hasPermi('system:album:remove')" )
    @Log(title = "家族相簿" , businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}" )
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(iTbFamilyAlbumService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
