package com.ruoyi.web.controller.api.shop;

import com.ruoyi.app.shop.goods.domain.TbGoods;
import com.ruoyi.app.shop.goods.service.ITbGoodsService;
import com.ruoyi.app.shop.type.domain.TbGoodsType;
import com.ruoyi.app.shop.type.service.ITbGoodsTypeService;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * 商品
 *
 * @author lishuwen
 * @date 2020-10-02
 */
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/system/shop/goods")
public class TbGoodsController extends BaseController {

    private final ITbGoodsService iTbGoodsService;

    private final ITbGoodsTypeService typeService;

    /**
     * 查询商品列表
     */
    @PreAuthorize("@ss.hasPermi('system:goods:list')")
    @GetMapping("/list")
    public TableDataInfo list(TbGoods tbGoods) {
        startPage();
        List<TbGoods> list = iTbGoodsService.selectTbGoodsList(tbGoods);
        return getDataTable(list);
    }

    /**
     * 商品分类
     */
    @GetMapping("/typeList")
    public AjaxResult typeList() {
        List<TbGoodsType> list = typeService.typeList();
        return AjaxResult.success(list);
    }

    /**
     * 获取商品详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:goods:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(iTbGoodsService.getById(id));
    }

    /**
     * 新增商品
     */
    @PreAuthorize("@ss.hasPermi('system:goods:add')")
    @Log(title = "商品", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TbGoods tbGoods) {
        tbGoods.setCreateTime(new Date());
        return toAjax(iTbGoodsService.save(tbGoods) ? 1 : 0);
    }

    /**
     * 修改商品
     */
    @PreAuthorize("@ss.hasPermi('system:goods:edit')")
    @Log(title = "商品", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TbGoods tbGoods) {
        return toAjax(iTbGoodsService.updateById(tbGoods) ? 1 : 0);
    }

    /**
     * 删除商品
     */
    @PreAuthorize("@ss.hasPermi('system:goods:remove')")
    @Log(title = "商品", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(iTbGoodsService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
