package com.ruoyi.web.controller.api.shop;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ruoyi.app.shop.type.domain.TbGoodsType;
import com.ruoyi.app.shop.type.service.ITbGoodsTypeService;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * 商品类型
 *
 * @author lishuwen
 * @date 2020-10-02
 */
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/system/shop/type")
public class TbGoodsTypeController extends BaseController {

    private final ITbGoodsTypeService iTbGoodsTypeService;

    /**
     * 查询商品类型列表
     */
    @PreAuthorize("@ss.hasPermi('system/shop:type:list')")
    @GetMapping("/list")
    public TableDataInfo list(TbGoodsType tbGoodsType) {
        startPage();
        LambdaQueryWrapper<TbGoodsType> lqw = new LambdaQueryWrapper<TbGoodsType>();
        if (StringUtils.isNotBlank(tbGoodsType.getName())) {
            lqw.like(TbGoodsType::getName, tbGoodsType.getName());
        }
        if (StringUtils.isNotBlank(tbGoodsType.getEndTime())) {
            lqw.between(TbGoodsType::getCreateTime, tbGoodsType.getBeginTime(), tbGoodsType.getEndTime());
        }
        List<TbGoodsType> list = iTbGoodsTypeService.list(lqw);
        return getDataTable(list);
    }

    /**
     * 获取商品类型详细信息
     */
    @PreAuthorize("@ss.hasPermi('system/shop:type:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(iTbGoodsTypeService.getById(id));
    }

    /**
     * 新增商品类型
     */
    @PreAuthorize("@ss.hasPermi('system/shop:type:add')")
    @Log(title = "商品类型", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TbGoodsType tbGoodsType) {
        tbGoodsType.setCreateTime(new Date());
        return toAjax(iTbGoodsTypeService.save(tbGoodsType) ? 1 : 0);
    }

    /**
     * 修改商品类型
     */
    @PreAuthorize("@ss.hasPermi('system/shop:type:edit')")
    @Log(title = "商品类型", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TbGoodsType tbGoodsType) {
        return toAjax(iTbGoodsTypeService.updateById(tbGoodsType) ? 1 : 0);
    }

    /**
     * 删除商品类型
     */
    @PreAuthorize("@ss.hasPermi('system/shop:type:remove')")
    @Log(title = "商品类型", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(iTbGoodsTypeService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
