package com.ruoyi.web.controller.system;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

import java.util.List;
import java.util.Arrays;

import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.TTemplatePage;
import com.ruoyi.system.service.ITTemplatePageService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 个人页模板Controller
 * 
 * @author ruoyi
 * @date 2020-09-16
 */
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/system/personal/template" )
public class TTemplatePageController extends BaseController {

    private final ITTemplatePageService iTTemplatePageService;

    /**
     * 查询个人页模板列表
     */
    @PreAuthorize("@ss.hasPermi('system:personal:template:list')")
    @GetMapping("/list")
    public TableDataInfo list(TTemplatePage tTemplatePage)
    {
        startPage();
        LambdaQueryWrapper<TTemplatePage> lqw = new LambdaQueryWrapper<TTemplatePage>();
        if (StringUtils.isNotBlank(tTemplatePage.getTitle())){
            lqw.eq(TTemplatePage::getTitle ,tTemplatePage.getTitle());
        }
        if (StringUtils.isNotBlank(tTemplatePage.getRoute())){
            lqw.eq(TTemplatePage::getRoute ,tTemplatePage.getRoute());
        }
        if (tTemplatePage.getEnable() != null){
            lqw.eq(TTemplatePage::getEnable ,tTemplatePage.getEnable());
        }
        List<TTemplatePage> list = iTTemplatePageService.list(lqw);
        return getDataTable(list);
    }

    /**
     * 导出个人页模板列表
     */
    @PreAuthorize("@ss.hasPermi('system:personal:template:export')" )
    @Log(title = "个人页模板" , businessType = BusinessType.EXPORT)
    @GetMapping("/export" )
    public AjaxResult export(TTemplatePage tTemplatePage) {
        LambdaQueryWrapper<TTemplatePage> lqw = new LambdaQueryWrapper<TTemplatePage>(tTemplatePage);
        List<TTemplatePage> list = iTTemplatePageService.list(lqw);
        ExcelUtil<TTemplatePage> util = new ExcelUtil<TTemplatePage>(TTemplatePage. class);
        return util.exportExcel(list, "page" );
    }

    /**
     * 获取个人页模板详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:personal:template:query')" )
    @GetMapping(value = "/{id}" )
    public AjaxResult getInfo(@PathVariable("id" ) Long id) {
        return AjaxResult.success(iTTemplatePageService.getById(id));
    }

    /**
     * 新增个人页模板
     */
    @PreAuthorize("@ss.hasPermi('system:personal:template:add')" )
    @Log(title = "个人页模板" , businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TTemplatePage tTemplatePage) {
        return toAjax(iTTemplatePageService.save(tTemplatePage) ? 1 : 0);
    }

    /**
     * 修改个人页模板
     */
    @PreAuthorize("@ss.hasPermi('system:personal:template:edit')" )
    @Log(title = "个人页模板" , businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TTemplatePage tTemplatePage) {
        return toAjax(iTTemplatePageService.updateById(tTemplatePage) ? 1 : 0);
    }

    /**
     * 状态修改
     */
    @PreAuthorize("@ss.hasPermi('system:personal:template:edit')")
    @PutMapping("/changeEnable")
    public AjaxResult changeStatus(@RequestBody TTemplatePage tTemplatePage)
    {
        if (tTemplatePage == null){
            return AjaxResult.error("数据不能为空");
        }
        if (tTemplatePage.getId() <= 0){
            return AjaxResult.error("ID不能为空");
        }
        if (!tTemplatePage.getEnable().equals("0") && !tTemplatePage.getEnable().equals("1")){
            return AjaxResult.error("修改失败");
        }
        tTemplatePage.setUpdateBy(SecurityUtils.getUsername());
        return toAjax(iTTemplatePageService.updateById(tTemplatePage) ? 1 : 0);
    }
    /**
     * 删除个人页模板
     */
    @PreAuthorize("@ss.hasPermi('system:personal:template:remove')" )
    @Log(title = "个人页模板" , businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}" )
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(iTTemplatePageService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
