package com.ruoyi.app.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ruoyi.app.controller.base.AppBaseController;
import com.ruoyi.app.domain.TbAppUser;
import com.ruoyi.app.domain.vo.AppMemberVo;
import com.ruoyi.app.service.ITbAppUserService;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 会员管理
 *
 * @author Alex
 * @date 2020-09-24
 */
@Api(value = "会员管理", tags = "会员管理")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/app/user")
public class AppUserController extends AppBaseController {

    private final ITbAppUserService userService;

    /**
     * 查询会员列表
     */
    @ApiOperation("查询会员列表")
    @GetMapping("/list")
    public AjaxResult memberList(AppMemberVo memberVo) {
        memberVo.setEnable("1");
        List<AppMemberVo> list = userService.selectAppMembers(memberVo.getEnable(), memberVo.getSearchValue(),
                memberVo.getBeginTime(), memberVo.getEndTime());
        return AjaxResult.success(list);
    }

    @ApiOperation("获取会员信息")
    @GetMapping("/get")
    public AjaxResult get(Long appUserId) {
        if (appUserId == null) {
            return AjaxResult.error("会员id不能为空");
        }
        return AjaxResult.success(userService.getAppMember(appUserId.toString(), null));
    }

    @ApiOperation("验证手机号是否存在")
    @PutMapping("/checkPhone")
    public AjaxResult checkPhone(@RequestBody AppMemberVo memberVo) {
        if (StringUtils.isBlank(memberVo.getMobile())) {
            return AjaxResult.error("手机号不能为空");
        }
        TbAppUser user = userService.getOne(new LambdaQueryWrapper<TbAppUser>()
                .eq(TbAppUser::getMobile, memberVo.getMobile())
        );
        if (user != null) {
            return AjaxResult.error("该手机号已经绑定");
        }
        return AjaxResult.success();
    }

    /**
     * 修改会员+家族成员
     */
    @ApiOperation("修改会员信息")
    @PutMapping("/editAppUser")
    public AjaxResult editAppUser(@RequestBody AppMemberVo memberVo) {
        if (memberVo.getMemberId() == null) {
            return AjaxResult.error("memberId不能为空");
        }
        if (memberVo.getUserId() == null) {
            return AjaxResult.error("userId不能为空");
        }
        return toAjax(userService.saveOrUpdate(memberVo) ? 1 : 0);
    }
}
