package com.ruoyi.app.controller;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ruoyi.app.controller.base.AppBaseController;
import com.ruoyi.app.domain.TbFamily;
import com.ruoyi.app.domain.TbMemberMiddle;
import com.ruoyi.app.domain.vo.AppFamilyVo;
import com.ruoyi.app.domain.vo.AppMemberVo;
import com.ruoyi.app.service.ITbFamilyService;
import com.ruoyi.app.service.ITbMemberMiddleService;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.utils.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * 家族管理
 *
 * @author Alex
 * @date 2020-09-24
 */
@Api(value = "家族管理",tags = "家族管理")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/app/family" )
public class FamilyController extends AppBaseController {

    private final ITbFamilyService familyService;
    private final ITbMemberMiddleService middleService;

    /**
     * 查询家族列表
     */
    @ApiOperation("查询家族列表")
    @GetMapping("/list")
    public TableDataInfo list(TbFamily tbFamily)
    {
        startPage();
        LambdaQueryWrapper<TbFamily> lqw = new LambdaQueryWrapper<>();
        if (StringUtils.isNotBlank(tbFamily.getCode())){
            lqw.eq(TbFamily::getCode ,tbFamily.getCode());
        }
        if (StringUtils.isNotBlank(tbFamily.getName())){
            lqw.like(TbFamily::getName ,tbFamily.getName());
        }
        if (StringUtils.isNotBlank(tbFamily.getFullName())){
            lqw.like(TbFamily::getFullName ,tbFamily.getFullName());
        }
        if (StringUtils.isNotBlank(tbFamily.getHometown())){
            lqw.eq(TbFamily::getHometown ,tbFamily.getHometown());
        }
        if (StringUtils.isNotBlank(tbFamily.getAddress())){
            lqw.eq(TbFamily::getAddress ,tbFamily.getAddress());
        }
        if (StringUtils.isNotBlank(tbFamily.getCity())){
            lqw.eq(TbFamily::getCity ,tbFamily.getCity());
        }
        List<AppFamilyVo> list = familyService.pageList(lqw);
        return getDataTable(list);
    }
    /**
     * 查询我的默认家族
     */
    @ApiOperation("我的默认家族")
    @GetMapping("/myFamily")
    public AjaxResult myFamily(){
        AppMemberVo memberVo = getLoginUser().getUser();
        Long memberId = memberVo.getMemberId();
        if (memberId == null) {
            return AjaxResult.error("成员id不能为空");
        }
        TbFamily family = familyService.myFamily(memberId);
        return AjaxResult.success(family);
    }

    @ApiOperation("我的家族列表")
    @GetMapping("/myList")
    public AjaxResult myList(){
        AppMemberVo memberVo = getLoginUser().getUser();
        Long memberId = memberVo.getMemberId();
        if (memberId == null) {
            return AjaxResult.error("成员id不能为空");
        }
        return AjaxResult.success(familyService.selectByMemberId(memberId));
    }

    /**
     * 创建家族
     */
    @ApiOperation("创建家族")
    @PostMapping("/create")
    @Transactional(rollbackFor = Exception.class)
    public AjaxResult add(@RequestBody TbFamily tbFamily) {
        AppMemberVo memberVo = getLoginUser().getUser();
        Date date = new Date();
        tbFamily.setCreateTime(date);
        tbFamily.setUpdateTime(date);
        if(!familyService.saveFamily(tbFamily)) {
            return AjaxResult.error("创建失败");
        }
        // 设置添加人为管理员
        TbMemberMiddle middle = new TbMemberMiddle();
        middle.setMemberId(memberVo.getMemberId());
        middle.setFamilyId(tbFamily.getId());
        middle.setPatriarch("Y");
        middle.setAdmin("Y");
        middle.setStatus("2");
        if(!middleService.save(middle)) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return AjaxResult.error("创建失败");
        }
        return AjaxResult.success("创建成功");
    }

    @ApiOperation("切换家族")
    @ApiImplicitParam(name = "familyId", value = "切换的家族ID",paramType="Long")
    @GetMapping("/change")
    public AjaxResult change(Long familyId) {
        AppMemberVo memberVo = getLoginUser().getUser();
        Long memberId = memberVo.getMemberId();
        if(familyService.change(memberId,familyId)){
            return AjaxResult.success();
        }
        return AjaxResult.error("切换失败");
    }

    /**
     * 申请加入家族
     * @param familyId
     * @return
     */
    @ApiOperation("申请加入家族")
    @ApiImplicitParam(name = "familyId", value = "申请的家族ID",paramType="Long")
    @GetMapping("/join")
    public AjaxResult join(Long familyId) {
        if (familyId == null) {
            return AjaxResult.error("家族id不能为空");
        }
        AppMemberVo memberVo = getLoginUser().getUser();
        memberVo.setFamilyId(familyId);
        return familyService.applyFamily(memberVo);
    }
    /**
     * 扫码申请加入家族
     * @param code
     * @return
     */
    @ApiOperation("扫码申请加入家族")
    @ApiImplicitParam(name = "code", value = "申请的家族二维码",paramType="String")
    @GetMapping("/scan")
    public AjaxResult scan(String code) {
        if (StringUtils.isBlank(code)) {
            return AjaxResult.error("二维码为空");
        }
        AppMemberVo memberVo = getLoginUser().getUser();
        TbFamily family = familyService.getOne(new LambdaQueryWrapper<TbFamily>()
                .eq(TbFamily::getCode,code)
                .last("limit 1")
        );
        if (family == null){
            return AjaxResult.error("家族不存在");
        }
        memberVo.setFamilyId(family.getId());
        return familyService.applyFamily(memberVo);
    }


    @ApiOperation("申请列表")
    @ApiImplicitParam(name = "familyId", value = "家族id",paramType="Long")
    @GetMapping("/selectApply")
    public AjaxResult selectApply(Long familyId){
        if (familyId == null){
            return AjaxResult.error("家族id不能为空");
        }
//        pageNum = pageNum == null ? 1 : pageNum;
//        pageSize = pageSize == null ? 10 : pageSize;
//        PageHelper.startPage(pageNum, pageSize, "mi.id desc");
        // 获取申请列表 1申请 3拒绝 4退出
        List<TbMemberMiddle> list = middleService.selectApplyMember(new LambdaQueryWrapper<TbMemberMiddle>()
                .eq(TbMemberMiddle::getFamilyId,familyId)
                .in(TbMemberMiddle::getStatus, Arrays.asList("1","3","4"))
        );
        return AjaxResult.success(list);
    }

    @ApiOperation("通过/拒绝申请")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "memberId", value = "申请的成员id",paramType="Long"),
            @ApiImplicitParam(name = "familyId", value = "家族id",paramType="Long"),
            @ApiImplicitParam(name = "status", value = "申请状态 2同意 3拒绝",paramType="String")
    })
    @GetMapping("/pass")
    public AjaxResult pass(Long memberId,Long familyId, String status){
        TbMemberMiddle middle = middleService.getOne(new LambdaQueryWrapper<TbMemberMiddle>()
                .eq(TbMemberMiddle::getFamilyId,familyId)
                .eq(TbMemberMiddle::getMemberId,memberId)
                .eq(TbMemberMiddle::getStatus,"1")
                .last("limit 1")
        );
        if (middle == null) {
            return AjaxResult.error("查无申请信息");
        }
        // 通过/拒绝申请
        middle.setStatus(status);
        if(!middleService.updateById(middle)){
            return AjaxResult.error("通过失败");
        }
        return AjaxResult.success("申请通过");
    }

    /**
     * 退出家族
     * @param familyId
     * @return
     */
    @ApiOperation("退出家族")
    @ApiImplicitParam(name = "familyId", value = "申请的家族ID",paramType="Long")
    @GetMapping("/quit")
    @Transactional(rollbackFor = Exception.class)
    public AjaxResult quit(Long familyId) {
        if (familyId == null) {
            return AjaxResult.error("家族id为空");
        }
        AppMemberVo user = getLoginUser().getUser();
        TbMemberMiddle middle = middleService.getOne(new LambdaQueryWrapper<TbMemberMiddle>()
                .eq(TbMemberMiddle::getMemberId,user.getMemberId())
                .eq(TbMemberMiddle::getFamilyId,familyId)
                .last("limit 1")
        );
        if (middle == null){
            return AjaxResult.error("未加入家族");
        }
        if ("Y".equals(middle.getPatriarch())) {
            // 创建者退出家族，则家族解散
            // 删除家族成员
            boolean ck = middleService.remove(new LambdaQueryWrapper<TbMemberMiddle>()
                    .eq(TbMemberMiddle::getFamilyId,familyId)
            );

            if (ck) {
                // 删除家族
                familyService.removeById(familyId);
                return AjaxResult.success("解散成功");
            }
            return AjaxResult.error("解散失败");
        }
        if ("4".equals(middle.getStatus())) {
            return AjaxResult.error("已退出");
        }
        middle.setStatus("4"); //退出
        middle.setAcquiesce("N"); //取消默认
        middle.setAdmin("N"); // 取消管理员
        if(!middleService.updateById(middle)){
            return AjaxResult.error("退出失败");
        }
        return AjaxResult.success("退出成功");
    }


    /**
     * 获取家族详细信息
     */
    @ApiOperation("获取家族详细信息")
    @ApiImplicitParam(name = "id", value = "主键id",paramType="Long")
    @GetMapping(value = "/getById/{id}" )
    public AjaxResult getInfo(@PathVariable Long id) {
        if (id == null){
            return AjaxResult.error("id不能为空");
        }
        LambdaQueryWrapper<TbFamily> lqw = new LambdaQueryWrapper<>();
        lqw.eq(TbFamily::getId ,id);
        List<AppFamilyVo> list = familyService.pageList(lqw);
        if (list.size() <= 0) {
            return AjaxResult.success();
        }
        AppFamilyVo family = list.get(0);
        return AjaxResult.success(family);
    }

    /**
     * 获取家族申请数量
     */
    @ApiOperation("获取家族申请数量")
    @ApiImplicitParam(name = "familyId", value = "家族id",paramType="Long")
    @GetMapping(value = "/getApplyCount" )
    public AjaxResult getApplyCount(Long familyId) {

        if (familyId == null){
            return AjaxResult.error("家族id不能为空");
        }
        return AjaxResult.success(familyService.getApplyCount(familyId));
    }

}
