package com.ruoyi.app.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ruoyi.app.domain.TbFeedbackinfo;
import com.ruoyi.app.service.ITbFeedbackinfoService;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.ServletUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.service.TokenService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * 信息反馈
 *
 * @author liuhj
 * @date 2020-10-20
 */
@Api(value = "信息反馈", tags = "信息反馈")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/app/feedbackinfo" )
public class FeedbackinfoController extends BaseController {

    private final ITbFeedbackinfoService iTbFeedbackinfoService;
    @Autowired
    private TokenService tokenService;

    /**
     * 查询信息反馈列表
     */
    @ApiOperation(value = "信息反馈列表", notes = "信息反馈列表")
    @GetMapping("/list")
    public TableDataInfo list(TbFeedbackinfo tbFeedbackinfo)
    {
        startPage();
        LambdaQueryWrapper<TbFeedbackinfo> lqw = new LambdaQueryWrapper<TbFeedbackinfo>();
        if (StringUtils.isNotBlank(tbFeedbackinfo.getTitle())){
            lqw.like(TbFeedbackinfo::getTitle ,tbFeedbackinfo.getTitle());
        }
        if (tbFeedbackinfo.getCreateBy() != null){
            lqw.eq(TbFeedbackinfo::getCreateBy ,tbFeedbackinfo.getCreateBy());
        }
        List<TbFeedbackinfo> list = iTbFeedbackinfoService.list(lqw);
        return getDataTable(list);
    }

    /**
     * 获取信息反馈详细信息
     */
    @ApiOperation(value = "信息反馈详细信息", notes = "信息反馈详细信息")
    @GetMapping(value = "/{id}" )
    public AjaxResult detail(@PathVariable("id" ) Long id) {
        return AjaxResult.success(iTbFeedbackinfoService.getById(id));
    }

    /**
     * 新增信息反馈
     */
    @ApiOperation("新增信息反馈")
    @Log(title = "信息反馈" , businessType = BusinessType.INSERT)
    @PostMapping("/add")
    public AjaxResult add(@RequestBody TbFeedbackinfo tbFeedbackinfo) {
        tbFeedbackinfo.setCreateTime(new Date());
        return toAjax(iTbFeedbackinfoService.save(tbFeedbackinfo) ? 1 : 0);
    }

    /**
     * 修改信息反馈
     */
    @ApiOperation("修改信息反馈")
    @Log(title = "信息反馈" , businessType = BusinessType.UPDATE)
    @PutMapping("/update")
    public AjaxResult edit(@RequestBody TbFeedbackinfo tbFeedbackinfo) {

        Date date = new Date();
        tbFeedbackinfo.setUpdateTime(date);
        return toAjax(iTbFeedbackinfoService.updateById(tbFeedbackinfo) ? 1 : 0);
    }

    /**
     * 删除信息反馈
     */
    @ApiOperation("删除信息反馈")
    @Log(title = "信息反馈" , businessType = BusinessType.DELETE)
    @DeleteMapping("/del")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(iTbFeedbackinfoService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
