package com.ruoyi.app.controller;


import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.pagehelper.PageHelper;
import com.ruoyi.app.annotation.LoginAppUser;
import com.ruoyi.app.base.AppLoginUser;
import com.ruoyi.app.controller.base.AppBaseController;
import com.ruoyi.app.domain.TbAppUser;
import com.ruoyi.app.domain.TbMyFriends;
import com.ruoyi.app.domain.vo.AppMemberVo;
import com.ruoyi.app.service.ITbAppUserService;
import com.ruoyi.app.service.ITbMyFriendsService;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.utils.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.net.Inet4Address;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * 我的好友
 *
 * @author Alex
 * @date 2020-10-11
 */
@Api(value = "我的好友",tags = "我的好友")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/app/friend" )
public class MyFriendController extends AppBaseController {

    private final ITbMyFriendsService friendsService;
    private final ITbAppUserService userService;


    /**
     * 查询我的好友列表
     */
    @ApiOperation("好友列表")
    @ApiImplicitParam(name = "searchValue", value = "模糊条件：好友ID、好友昵称、拼音大写首字母",paramType="String")
    @GetMapping("/myList")
    public AjaxResult list(String searchValue) {
        AppMemberVo user = getLoginUser().getUser();
        TbMyFriends tbMyFriends = new TbMyFriends();
        //已通过的好友
        tbMyFriends.setAppUserId(user.getUserId());
        tbMyFriends.setStatus("2");
        tbMyFriends.setSearchValue(searchValue);
        List<TbMyFriends> list = friendsService.pageList(tbMyFriends);
        return AjaxResult.success(list);
    }


    @ApiOperation("添加好友-搜索")
    @ApiImplicitParam(name = "searchValue", value = "查询条件：会员号/手机号",paramType="String")
    @GetMapping("/getByMobile")
    public AjaxResult getByMobile(String searchValue) {
        if (StringUtils.isBlank(searchValue)){
            return AjaxResult.error("会员号/手机号不能为空");
        }
        return AjaxResult.success(userService.getAppMember(null,searchValue));
    }


    @ApiOperation("根据id获取好友")
    @ApiImplicitParam(name = "id", value = "好友id",paramType="Long")
    @GetMapping("/getById")
    public AjaxResult get(Long id) {
        if (id == null){
            return AjaxResult.error("id不能为空");
        }
        return AjaxResult.success(userService.getAppMember(id.toString(),null));
    }


    @ApiOperation("申请添加好友")
    @PostMapping("/add")
    public AjaxResult add(@RequestBody TbMyFriends friends) {
        if (friends.getFriendsUserId() == null) {
            return AjaxResult.error("好友id不能为空");
        }
        // 当前登录人
        AppMemberVo user = getLoginUser().getUser();
        friends.setAppUserId(user.getUserId());
        if (user.getUserId().equals(friends.getFriendsUserId())) {
            return AjaxResult.error("不能加自己为好友");
        }
        // 查询是否已经存在
        TbMyFriends item = friendsService.getOne(new LambdaQueryWrapper<TbMyFriends>()
                .nested(
                        i -> i.eq(TbMyFriends::getAppUserId, friends.getAppUserId())
                        .eq(TbMyFriends::getFriendsUserId, friends.getFriendsUserId())
                )
                .or(
                        i -> i.eq(TbMyFriends::getAppUserId,friends.getFriendsUserId())
                        .eq(TbMyFriends::getFriendsUserId,friends.getAppUserId())
                )
                .last("limit 1")
        );
        // 已存在并已通过
        if (item != null && item.getStatus().equals("2")) {
            return AjaxResult.success("好友已添加");
        }
        friends.setStatus("1"); //申请
        friends.setCreateBy(user.getUserId().toString());
        friends.setCreateTime(new Date());
        // 不存在则新增
        if (item == null) {
            friendsService.save(friends);
        }else{
            // 已存在未通过
            friends.setId(item.getId());
            friendsService.updateById(friends);
        }
        return AjaxResult.success();
    }


    @ApiOperation("申请列表")
    @GetMapping("/selectApply")
    public AjaxResult selectApply() {
        AppMemberVo loginUser = getLoginUser().getUser();
        Long appUserId = loginUser.getUserId();
        List<TbMyFriends> list = friendsService.selectApply(new LambdaQueryWrapper<TbMyFriends>()
                .eq(TbMyFriends::getFriendsUserId, appUserId)
                .in(TbMyFriends::getStatus, Arrays.asList("1","3"))
        );
        return AjaxResult.success(list);
    }


    @ApiOperation("通过/拒绝申请")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "好友id",paramType="Long"),
            @ApiImplicitParam(name = "status", value = "状态 1申请 2已通过 3未通过",paramType="String")
    })
    @GetMapping("/pass")
    public AjaxResult pass(Long id, String status){
        if (id == null) {
            return AjaxResult.error("好友id不能为空");
        }
        AppMemberVo user = getLoginUser().getUser();
        TbMyFriends friends = friendsService.getByUid(id, user.getUserId());
        if (friends == null) {
            return AjaxResult.error("该用户已不存在");
        }
        if ("2".equals(friends.getStatus())) {
            return AjaxResult.error("好友已添加");
        }
        friends.setStatus(status);
        friends.setUpdateBy(user.getUserId().toString());
        friends.setUpdateTime(new Date());
        return toAjax(friendsService.updateById(friends) ? 1 : 0);
    }

    @ApiOperation("删除好友")
    @ApiImplicitParam(name = "id", value = "好友id",paramType="Long")
    @DeleteMapping("/del/{id}")
    public AjaxResult del(@PathVariable Long id) {
        if (id == null) {
            return AjaxResult.error("id不能为空");
        }
        return toAjax(friendsService.removeById(id) ? 1 : 0);
    }

    /**
     * 获取好友申请数量
     */
    @ApiOperation("获取好友申请数量")
    @GetMapping(value = "/getApplyCount")
    public AjaxResult getApplyCount() {
        AppMemberVo loginUser = getLoginUser().getUser();
        Long appUserId = loginUser.getUserId();
        return AjaxResult.success(friendsService.getApplyCount(appUserId));
    }

}
