package com.ruoyi.app.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ruoyi.app.annotation.LoginAppUser;
import com.ruoyi.app.controller.base.AppBaseController;
import com.ruoyi.app.domain.TbAppUser;
import com.ruoyi.app.domain.TbMyProfile;
import com.ruoyi.app.domain.TbProfileImg;
import com.ruoyi.app.domain.TbPublishImg;
import com.ruoyi.app.domain.vo.AppMemberVo;
import com.ruoyi.app.service.ITbMyProfileService;
import com.ruoyi.app.service.ITbProfileImgService;
import com.ruoyi.app.service.ITbPublishImgService;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 个人详情
 *
 * @author Alex
 * @date 2020-10-11
 */
@Api(value = "个人详情", tags = "个人详情")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/app/profile")
public class MyProfileController extends AppBaseController {

    private final ITbMyProfileService profileService;
    private final ITbProfileImgService profileImgService;

    @ApiOperation("个人详情列表")
    @ApiImplicitParam(name = "appUserId", value = "会员ID", paramType = "Long")
    @GetMapping("/list")
    public AjaxResult list(Long appUserId) {
        if (appUserId == null) {
            return AjaxResult.error("参数为空");
        }
        List<TbMyProfile> profileList = profileService.list(new LambdaQueryWrapper<TbMyProfile>()
                .eq(TbMyProfile::getAppUserId, appUserId)
        );
        if (profileList.size() > 0) {
            List<Long> ids = new ArrayList<>();
            profileList.forEach(item -> {
                ids.add(item.getId());
            });
            List<TbProfileImg> imgList = profileImgService.list(new LambdaQueryWrapper<TbProfileImg>()
                    .in(TbProfileImg::getProfileId, ids)
            );
            if (imgList.size() > 0) {
                profileList.forEach(item -> {
                    List<TbProfileImg> newList = new ArrayList<>();
                    imgList.forEach(img -> {
                        if (item.getId().equals(img.getProfileId())) {
                            newList.add(img);
                        }
                    });
                    item.setImgList(newList);
                });
            }
        }
        return AjaxResult.success(profileList);
    }

    @ApiOperation("添加/修改个人详情")
    @PostMapping("/saveOrUpdate")
    public AjaxResult saveOrUpdate(@RequestBody TbMyProfile profile) {
        if (StringUtils.isBlank(profile.getTitle()) || StringUtils.isBlank(profile.getCotents())) {
            return AjaxResult.error("标题或内容不能为空");
        }
        if (profile.getAppUserId() == null) {
            return AjaxResult.error("参数异常");
        }
        AppMemberVo user = getLoginUser().getUser();
        Date date = new Date();
        //新增
        if (profile.getId() == null) {
            profile.setCreateBy(user.getUserId());
            profile.setCreateTime(date);
        } else {
            // 修改
            profile.setUpdateBy(user.getUserId());
            profile.setUpdateTime(date);
        }
        profileService.saveOrUpdate(profile);
        //先删除，在重新添加图片
        profileImgService.remove(new LambdaQueryWrapper<TbProfileImg>()
                .eq(TbProfileImg::getProfileId, profile.getId())
        );
        if (profile.getImgList().size() > 0) {
            List<TbProfileImg> imgList = profile.getImgList();
            imgList.forEach(item -> {
                item.setProfileId(profile.getId());
            });
            profileImgService.saveBatch(imgList);
        }
        return AjaxResult.success();
    }

    @ApiOperation("获取个人详情")
    @ApiImplicitParam(name = "id", value = "主键id", paramType = "Long")
    @GetMapping("/get")
    public AjaxResult get(Long id) {
        if (id == null) {
            return AjaxResult.error("id不能为空");
        }
        TbMyProfile profile = profileService.getById(id);
        if (profile == null) {
            return AjaxResult.success();
        }
        // 图片列表
        List<TbProfileImg> imgList = profileImgService.list(new LambdaQueryWrapper<TbProfileImg>()
                .eq(TbProfileImg::getProfileId, profile.getId())
        );
        if (imgList.size() > 0) {
            profile.setImgList(imgList);
        }
        return AjaxResult.success(profile);
    }

    @ApiOperation("删除个人详情")
    @ApiImplicitParam(name = "id", value = "主键id", paramType = "Long")
    @GetMapping("/del")
    public AjaxResult del(Long id) {
        if (id == null) {
            return AjaxResult.error("id不能为空");
        }
        return toAjax(profileService.removeById(id) ? 1 : 0);
    }

    @ApiOperation("删除个人详情 图片")
    @ApiImplicitParam(name = "id", value = "图片id", paramType = "Long")
    @GetMapping("/delImg")
    public AjaxResult delImg(Long id) {
        if (id == null) {
            return AjaxResult.error("id不能为空");
        }
        return toAjax(profileImgService.removeById(id) ? 1 : 0);
    }

    @ApiOperation("获取个人详情 图片列表")
    @ApiImplicitParam(name = "profileId", value = "详情id", paramType = "Long")
    @GetMapping("/getImgByProfileId/{profileId}")
    public AjaxResult listImg(@PathVariable Long profileId) {
        if (profileId == null) {
            return AjaxResult.error("详情Id不能为空");
        }
        List<TbProfileImg> list = profileImgService.list(new LambdaQueryWrapper<TbProfileImg>()
                .eq(TbProfileImg::getProfileId, profileId)
        );
        return AjaxResult.success(list);
    }

}
