package com.ruoyi.app.controller;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.pagehelper.PageHelper;
import com.ruoyi.app.controller.base.AppBaseController;
import com.ruoyi.app.domain.TbAppUser;
import com.ruoyi.app.domain.TbMyPublish;
import com.ruoyi.app.domain.TbPublishImg;
import com.ruoyi.app.domain.vo.AppFamilyVo;
import com.ruoyi.app.domain.vo.AppMemberVo;
import com.ruoyi.app.service.ITbAppUserService;
import com.ruoyi.app.service.ITbMyPublishService;
import com.ruoyi.app.service.ITbPublishImgService;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.FileUploadUtils;
import com.ruoyi.framework.config.ServerConfig;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.aspectj.weaver.loadtime.Aj;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;

/**
 * 我的近况
 *
 * @author Alex
 * @date 2020-10-09
 */
@Api(value = "我的近况",tags = "我的近况")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/app/publish" )
public class MyPublishController extends AppBaseController {

    private final ITbMyPublishService publishService;
    private final ServerConfig serverConfig;
    private final ITbPublishImgService imgService;


    /**
     * 查询家族朋友圈
     * @return
     */
    @ApiOperation("族友圈 分页列表")
    @GetMapping("/selectCircle")
    public TableDataInfo selectCircle(Long familyId){
//        if (familyId == null) {
//            return new TableDataInfo();
//        }
        AppMemberVo user = getLoginUser().getUser();
        List<TbMyPublish> list = publishService.selectPublish(user.getUserId(), familyId);
        return getDataTable(list);
    }

    /**
     * 查询我的近况 列表
     */
    @ApiOperation("我的近况 分页列表")
    @ApiImplicitParam(name = "appUserId", value = "会员id",paramType="Long")
    @GetMapping("/list")
    public TableDataInfo pageList(Integer pageNum,Integer pageSize,Long appUserId) {
        TbMyPublish tbMyPublish = new TbMyPublish();
        tbMyPublish.setAppUserId(appUserId);
        tbMyPublish.setDeleted("N");

        pageNum = pageNum == null ? 1 : pageNum;
        pageSize = pageSize == null ? 10 : pageSize;
        PageHelper.startPage(pageNum, pageSize, "create_time desc");
        List<TbMyPublish> list = publishService.pageList(tbMyPublish);
        return getDataTable(list);
    }

    @ApiOperation("获取我的近况详情")
    @ApiImplicitParam(name = "id", value = "主键id",paramType="Long")
    @GetMapping("/get")
    public AjaxResult get(Long id) {
        if (id == null) {
            return AjaxResult.error("id不能为空");
        }
        AppMemberVo user = getLoginUser().getUser();
        return AjaxResult.success(publishService.getPublish(id,user.getUserId()));
    }

    @ApiOperation("上传图片")
    @ResponseBody
    @PostMapping("/upload")
    public AjaxResult upload(@RequestParam(value = "file") MultipartFile files[]) {

        // 上传文件路径
        String filePath = RuoYiConfig.getUploadPath()+"/publish";
        if (files != null && files.length >= 1) {
            try {
                AjaxResult ajax = AjaxResult.success();
                for (MultipartFile file : files) {
                    Map<String, Object> map = new HashMap<>(2);
                    // 上传并返回新文件名称
                    String fileName = FileUploadUtils.upload(filePath, file);
                    String url = serverConfig.getUrl() + fileName;
                    map.put("fileName", fileName);
                    map.put("url", url);
                    ajax.put("files", map);
                }
                return ajax;
            } catch (Exception e) {
                return AjaxResult.error(e.getMessage());
            }
        }
        return AjaxResult.success("上传成功");
    }

    @ApiOperation("新增我的近况")
    @PostMapping("/add")
    @Transactional(rollbackFor = Exception.class)
    public AjaxResult add(@RequestBody TbMyPublish publish) {
        if (StringUtils.isBlank(publish.getContents())) {
            return AjaxResult.error("内容不能为空");
        }
        if (publish.getContents().length() > 500) {
            return AjaxResult.error("发表内容不能超过500字");
        }
        AppMemberVo user = getLoginUser().getUser();
        publish.setAppUserId(user.getUserId());
        publish.setCreateBy(user.getUserId().toString());
        publish.setCreateTime(new Date());
        if (!publishService.save(publish)){
            return AjaxResult.error("发表失败");
        }

        // 插入图片列表
        List<TbPublishImg> imgList = publish.getImgList();
        if (imgList.size() <= 0) {
            return AjaxResult.success();
        }
        imgList.forEach(item -> {
            item.setPublishId(publish.getId());
        });
        if(imgService.saveBatch(imgList)){
            return AjaxResult.success();
        }
        return AjaxResult.error("发表失败");
    }


    /**
     * 逻辑删除我的近况
     */
    @ApiOperation("逻辑删除我的近况 ")
    @ApiImplicitParam(name = "ids", value = "主键id数组",paramType="Long")
    @DeleteMapping("/del/{ids}" )
    public AjaxResult remove(@PathVariable Long[] ids) {
        if (ids.length <= 0) {
            return AjaxResult.error("id不能为空");
        }
        return toAjax(publishService.delPublish(Arrays.asList(ids)) ? 1 : 0);
    }

    @ApiOperation("新增 族友的留言纪念")
    @PostMapping("/addMessage")
    @Transactional(rollbackFor = Exception.class)
    public AjaxResult addMessage(@RequestBody TbMyPublish publish) {
        if (publish.getAppUserId() == null) {
            return AjaxResult.error("留念的对象不能为空");
        }
        if (StringUtils.isBlank(publish.getContents())) {
            return AjaxResult.error("内容不能为空");
        }
        if (publish.getContents().length() > 500) {
            return AjaxResult.error("发表内容不能超过500字");
        }
        AppMemberVo user = getLoginUser().getUser();
        publish.setMessage("Y"); // 留念标志
        publish.setMessageUserId(user.getUserId());
        publish.setCreateBy(user.getUserId().toString());
        publish.setCreateTime(new Date());
        if (!publishService.save(publish)){
            return AjaxResult.error("发表失败");
        }

        // 插入图片列表
        List<TbPublishImg> imgList = publish.getImgList();
        if (imgList.size() <= 0) {
            return AjaxResult.success();
        }
        imgList.forEach(item -> {
            item.setPublishId(publish.getId());
        });
        if(imgService.saveBatch(imgList)){
            return AjaxResult.success();
        }
        return AjaxResult.error("发表失败");
    }

}
