package com.ruoyi.app.controller;

import com.ruoyi.app.annotation.PassToken;
import com.ruoyi.app.domain.TbNews;
import com.ruoyi.app.service.ITbNewsService;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 资讯
 *
 * @author lsw
 * @date 2020-10-12
 */
@Api(value = "新闻资讯", tags = "新闻资讯")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/app/news")
public class TbNewsController extends BaseController {

    private final ITbNewsService iTbNewsService;

    @PassToken
    @ApiOperation(value = "资讯列表", notes = "资讯列表")
    @GetMapping("/list")
    public TableDataInfo list() {
        startPage();
        List<TbNews> list = iTbNewsService.selectTbNewsList();
        return getDataTable(list);
    }

    @PassToken
    @ApiOperation(value = "资讯详情", notes = "资讯详情")
    @GetMapping(value = "/{id}")
    public AjaxResult detail(@PathVariable("id") Long id) {
        TbNews news = iTbNewsService.getById(id);
        if (news != null && news.getState() == 0) {
            //每次浏览都新增一个浏览记录,性能不是很好
            news.setViews(news.getViews() + 1);
            iTbNewsService.updateById(news);
            return AjaxResult.success(news);
        }
        return AjaxResult.error("文章不存在或已删除");
    }
}
