package com.ruoyi.app.controller.shop;

import com.ruoyi.app.annotation.PassToken;
import com.ruoyi.app.shop.goods.domain.TbGoods;
import com.ruoyi.app.shop.goods.service.ITbGoodsService;
import com.ruoyi.app.shop.type.domain.TbGoodsType;
import com.ruoyi.app.shop.type.service.ITbGoodsTypeService;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 商品
 *
 * @author lishuwen
 * @date 2020-10-02
 */
@Api(value = "商品", tags = "商品")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/app/shop/goods")
public class TbGoodsController extends BaseController {

    private final ITbGoodsService iTbGoodsService;

    private final ITbGoodsTypeService typeService;

    @PassToken
    @ApiOperation(value = "商品列表", notes = "商品列表")
    @GetMapping("/list")
    public TableDataInfo list(TbGoods tbGoods) {
        startPage();
        tbGoods.setState(0);
        List<TbGoods> list = iTbGoodsService.selectTbGoodsList(tbGoods);
        return getDataTable(list);
    }

    @PassToken
    @ApiOperation(value = "商品分类", notes = "商品分类")
    @GetMapping("/typeList")
    public AjaxResult typeList() {
        List<TbGoodsType> list = typeService.typeList();
        return AjaxResult.success(list);
    }

    @PassToken
    @ApiOperation(value = "商品详细信息", notes = "商品详细信息")
    @GetMapping(value = "/{id}")
    public AjaxResult detail(@PathVariable("id") Long id) {
        TbGoods goods = iTbGoodsService.getById(id);
        if (goods != null && goods.getState() == 0) {
            return AjaxResult.success(goods);
        }
        return AjaxResult.error("商品不存在或已删除");
    }
}
