package com.ruoyi.app.interceptor;

import com.alibaba.fastjson.JSON;
import com.ruoyi.app.annotation.PassToken;
import com.ruoyi.app.base.AppLoginUser;
import com.ruoyi.app.base.AppTokenService;
import com.ruoyi.app.domain.TbAppUser;
import com.ruoyi.app.domain.vo.AppMemberVo;
import com.ruoyi.app.service.ITbAppUserService;
import com.ruoyi.common.constant.HttpStatus;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.ServletUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Method;


/**
 * token验证拦截
 * @author Alex
 */
public class AuthenticationInterceptor implements HandlerInterceptor {
    @Autowired
    ITbAppUserService userService;
    @Autowired
    AppTokenService appTokenService;
    @Override
    public boolean preHandle(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, Object object) throws Exception {
        String token = httpServletRequest.getHeader("token");// 从 http 请求头中取出 token
        // 如果不是映射到方法直接通过
        if(!(object instanceof HandlerMethod)){
            return true;
        }
        HandlerMethod handlerMethod=(HandlerMethod)object;
        Method method=handlerMethod.getMethod();
        //检查是否有passtoken注释，有则跳过认证
        if (method.isAnnotationPresent(PassToken.class)) {
            PassToken passToken = method.getAnnotation(PassToken.class);
            if (passToken.required()) {
                return true;
            }
        }
        // 执行认证
        if (token == null) {
            //throw new RuntimeException("无token，请重新登录");
            ServletUtils.renderString(httpServletResponse, JSON.toJSONString(AjaxResult.error(HttpStatus.UNAUTHORIZED, "未登陆，请先登录")));
            return false;
        }
        // 获取 token 中的 loginUser
        AppLoginUser loginUser = appTokenService.getLoginUser(token);
        if (loginUser == null) {
           // throw new RuntimeException("用户不存在，请重新登录");
            ServletUtils.renderString(httpServletResponse, JSON.toJSONString(AjaxResult.error(HttpStatus.UNAUTHORIZED, "未登陆，请先登录")));
            return false;
        }
        AppMemberVo user = loginUser.getUser();
        if (user == null) {
            ServletUtils.renderString(httpServletResponse, JSON.toJSONString(AjaxResult.error(HttpStatus.UNAUTHORIZED, "未登陆，请先登录")));
            return false;
            //throw new RuntimeException("用户不存在，请重新登录");
        }
        // 验证 token
        appTokenService.verifyToken(loginUser);
        return true;
    }

    @Override
    public void postHandle(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, Object o, ModelAndView modelAndView) throws Exception {

    }
    @Override
    public void afterCompletion(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, Object o, Exception e) throws Exception {

    }
}
