package com.ruoyi.app.pay;

import com.alipay.api.AlipayApiException;
import com.alipay.api.domain.AlipayTradeAppPayModel;
import com.alipay.api.internal.util.AlipaySignature;
import com.ijpay.alipay.AliPayApi;
import com.ijpay.alipay.AliPayApiConfig;
import com.ijpay.alipay.AliPayApiConfigKit;
import com.ijpay.core.kit.WxPayKit;
import com.ruoyi.app.annotation.PassToken;
import com.ruoyi.app.shop.orders.domain.TbOrders;
import com.ruoyi.app.shop.orders.service.ITbOrdersService;
import com.ruoyi.common.core.domain.AjaxResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.Map;

/**
 * 支付宝支付
 */
@RestController
@RequestMapping("/app/aliPay")
public class AliPayController extends AbstractAliPayApiController {

    @Autowired
    private AliPayBean aliPayBean;

    @Autowired
    ITbOrdersService service;
    /**
     * 普通公钥模式
     */
    private final static String NOTIFY_URL = "app/aliPay/notify_url";

    @Override
    public AliPayApiConfig getApiConfig() {
        AliPayApiConfig aliPayApiConfig;
        try {
            aliPayApiConfig = AliPayApiConfigKit.getApiConfig(aliPayBean.getAppId());
        } catch (Exception e) {
            aliPayApiConfig = AliPayApiConfig.builder()
                    .setAppId(aliPayBean.getAppId())
                    .setAliPayPublicKey(aliPayBean.getPublicKey())
                    .setAppCertPath(aliPayBean.getAppCertPath())
                    .setAliPayCertPath(aliPayBean.getAliPayCertPath())
                    .setAliPayRootCertPath(aliPayBean.getAliPayRootCertPath())
                    .setCharset("UTF-8")
                    .setPrivateKey(aliPayBean.getPrivateKey())
                    .setServiceUrl(aliPayBean.getServerUrl())
                    .setSignType("RSA2")
                    // 普通公钥方式
                    .build();

        }
        return aliPayApiConfig;
    }

    /**
     * app支付
     */
    @PostMapping(value = "/appPay")
    public AjaxResult appPay(@RequestBody TbOrders order) {
        if (order == null) {
            return AjaxResult.error("订单无效");
        }
        //用户第一次未支付成功，去我的订单支付,验证订单的有效性
        TbOrders check = service.selectTbOrdersByOrderNum(order.getOrderNum());
        if (check == null) {
            order.setPayType(1);
            order.setOrderNum(WxPayKit.generateStr());
        } else {
            order = check;
        }
        try {
            AlipayTradeAppPayModel model = new AlipayTradeAppPayModel();
            model.setBody(order.getTitle());
            model.setSubject(order.getTitle());
            model.setOutTradeNo(order.getOrderNum());
            model.setTimeoutExpress("30m");
            model.setTotalAmount("0.01");
            model.setPassbackParams("callback params");
            model.setProductCode("QUICK_MSECURITY_PAY");
            String orderInfo = AliPayApi.appPayToResponse(model, aliPayBean.getDomain() + NOTIFY_URL).getBody();

            // 保存订单信息*********************************************
            if (check == null) {
                order.setCreateTime(new Date());
                boolean flag = service.save(order);
                if (!flag) {
                    return AjaxResult.error("订单异常");
                }
            }
            return AjaxResult.success(orderInfo);
        } catch (AlipayApiException e) {
            e.printStackTrace();
            return AjaxResult.error(e.getMessage());
        }
    }

    @PassToken
    @RequestMapping(value = "/notify_url")
    @ResponseBody
    public String notifyUrl(HttpServletRequest request) {
        try {
            // 获取支付宝POST过来反馈信息
            Map<String, String> params = AliPayApi.toMap(request);
            String out_trade_no = params.get("out_trade_no");
            boolean verifyResult = AlipaySignature.rsaCheckV1(params, aliPayBean.getPublicKey(), "UTF-8", "RSA2");
            if (verifyResult) {
                // TODO 请在这里加上商户的业务逻辑程序代码 异步通知可能出现订单重复通知 需要做去重处理
                // 更新订单信息*********************************************
                TbOrders order = service.selectTbOrdersByOrderNum(out_trade_no);
                if (order != null && order.getState() == 0) {
                    order.setTransactionId(params.get("trade_no"));
                    order.setOpenId(params.get("buyer_logon_id"));
                    order.setState(1);
                    order.setPaySuccessTime(new Date());
                    service.updateById(order);
                }
                // 更新订单信息*********************************************
                return "success";
            } else {
                return "failure";
            }
        } catch (AlipayApiException e) {
            e.printStackTrace();
            return "failure";
        }
    }

}