
package com.ruoyi.app.resolver;

import com.ruoyi.app.annotation.LoginAppUser;
import com.ruoyi.app.base.AppTokenService;
import com.ruoyi.app.domain.TbAppUser;
import com.ruoyi.app.domain.vo.AppMemberVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.MethodParameter;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.support.WebDataBinderFactory;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;
import org.springframework.web.method.support.ModelAndViewContainer;

/**
 * 有@LoginAppUser注解的方法参数，注入当前登录用户
 *
 * @author luobo
 */
@Component
public class LoginAppUserHandlerMethodArgumentResolver implements HandlerMethodArgumentResolver {
    @Autowired
    private AppTokenService appTokenService;

    @Override
    public boolean supportsParameter(MethodParameter parameter) {
        //接收数据类型及注解类型
        return parameter.getParameterType().isAssignableFrom(AppMemberVo.class) && parameter.hasParameterAnnotation(LoginAppUser.class);
    }

    @Override
    public Object resolveArgument(MethodParameter parameter, ModelAndViewContainer container,
                                  NativeWebRequest request, WebDataBinderFactory factory) throws Exception {
        //直接从AppTokenService里获取登录用户信息
        return appTokenService.getLoginUser().getUser();
    }
}
