package com.ruoyi.app.family.photo.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.app.domain.TbFamily;
import com.ruoyi.app.family.album.domain.TbFamilyAlbum;
import com.ruoyi.app.family.album.service.ITbFamilyAlbumService;
import com.ruoyi.app.family.photo.domain.TbFamilyPhoto;
import com.ruoyi.app.family.photo.mapper.TbFamilyPhotoMapper;
import com.ruoyi.app.family.photo.service.ITbFamilyPhotoService;
import com.ruoyi.app.service.ITbFamilyService;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.exception.CustomException;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.FileUploadUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;


/**
 * 家族相簿照片
 *
 * @author luobo
 * @date 2020-10-14
 */
@Service
public class TbFamilyPhotoServiceImpl extends ServiceImpl<TbFamilyPhotoMapper, TbFamilyPhoto> implements ITbFamilyPhotoService {
    @Autowired
    private ITbFamilyAlbumService familyAlbumService;

    @Autowired
    private ITbFamilyService familyService;
    @Override
    public List<TbFamilyPhoto> queryList(TbFamilyPhoto entity) {
        LambdaQueryWrapper<TbFamilyPhoto> lqw = new LambdaQueryWrapper<TbFamilyPhoto>();
        if (entity.getFamilyId() != null){
            lqw.eq(TbFamilyPhoto::getFamilyId ,entity.getFamilyId());
        }
        if (entity.getAlbumId() != null){
            lqw.eq(TbFamilyPhoto::getAlbumId ,entity.getAlbumId());
        }
        if (StringUtils.isNotBlank(entity.getTitle())){
            lqw.eq(TbFamilyPhoto::getTitle ,entity.getTitle());
        }
        if (StringUtils.isNotBlank(entity.getUrl())){
            lqw.eq(TbFamilyPhoto::getUrl ,entity.getUrl());
        }
        if (StringUtils.isNotBlank(entity.getStatus())){
            lqw.eq(TbFamilyPhoto::getStatus ,entity.getStatus());
        }
        //时间倒序
        lqw.orderByDesc(TbFamilyPhoto::getCreateTime);

        List<TbFamilyPhoto> list = this.list(lqw);

        list.forEach(item->{
            TbFamily family = familyService.getById(item.getFamilyId());
            item.setFamily(family);
            TbFamilyAlbum album = familyAlbumService.getById(item.getAlbumId());
            item.setAlbum(album);
        });

        return list;
    }

    @Override
    public Integer getAlbumPhotoCount(Long albumId) {
        LambdaQueryWrapper<TbFamilyPhoto> queryWrapper = new LambdaQueryWrapper<TbFamilyPhoto>()
                .eq(TbFamilyPhoto::getAlbumId,albumId);
        return this.count(queryWrapper);
    }

    @Override
    public TbFamilyPhoto getEntityById(Long id) {
        TbFamilyPhoto entity = this.getById(id);
        if (entity!=null){
            TbFamily family = familyService.getById(entity.getFamilyId());
            entity.setFamily(family);
            TbFamilyAlbum album = familyAlbumService.getById(entity.getAlbumId());
            entity.setAlbum(album);
        }
        return entity;
    }

    @Override
    public Boolean batchUpload(Long userId, Long albumId, MultipartFile[] files, String modName) {
        //查询相簿信息
        TbFamilyAlbum albumEntity = familyAlbumService.getEntityById(albumId);
        if (albumEntity == null) {
            throw new CustomException("请选择相簿！");
        }
        try {

            // 上传文件路径
            String filePath = RuoYiConfig.getUploadPath();
            //@TODO 上传添加模块名称 luobo
            if (StringUtils.isNotEmpty(modName) && !modName.equalsIgnoreCase("null") && !modName.equalsIgnoreCase("undefined")) {
                filePath += "/" + modName;
            }
            List<TbFamilyPhoto> list = new ArrayList<>();
            for (MultipartFile item : files) {
                TbFamilyPhoto entity = new TbFamilyPhoto();
                entity.setAlbumId(albumEntity.getId());
                entity.setFamilyId(albumEntity.getFamilyId());
                String title = item.getOriginalFilename();
                title = title.substring(0, title.lastIndexOf("."));
                entity.setTitle(title);


                // 上传并返回新文件名称
                String fileName = FileUploadUtils.upload(filePath, item);
                entity.setUrl(fileName);
                entity.setCreateBy(userId);
                entity.setCreateTime(new Date());
                entity.setStatus("0");
                entity.setDelFlag("0");
                list.add(entity);
            }
            // 上传并返回新文件名称
            if (list.size() > 0) {
                this.saveBatch(list);
            }
            return true;
        } catch (Exception e) {
            throw new CustomException(e.getMessage());
        }

    }
}
