package com.ruoyi.app.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.app.domain.TbMemberMiddle;
import com.ruoyi.app.domain.vo.AppFamilyVo;
import com.ruoyi.app.domain.vo.AppMemberVo;
import com.ruoyi.app.service.ITbAppUserService;
import com.ruoyi.app.service.ITbFamilyMemberService;
import com.ruoyi.app.service.ITbMemberMiddleService;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.PushUtil;
import com.ruoyi.common.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.app.mapper.TbFamilyMapper;
import com.ruoyi.app.domain.TbFamily;
import com.ruoyi.app.service.ITbFamilyService;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 家族Service业务层处理
 *
 * @author Alex
 * @date 2020-09-28
 */
@Service
public class TbFamilyServiceImpl extends ServiceImpl<TbFamilyMapper, TbFamily> implements ITbFamilyService {

    @Autowired
    private ITbFamilyService familyService;
    @Autowired
    private ITbMemberMiddleService middleService;
    @Autowired
    private ITbAppUserService userService;
    @Autowired
    private ITbFamilyMemberService memberService;

    /**
     * 根据会员id获取家族列表
     * @param uid
     * @return
     */
    @Override
    public List<TbFamily> selectByUserId(Long uid) {
        return baseMapper.selectByUserId(uid);
    }
    /**
     * 根据成员id获取家族列表
     * @param mid
     * @return
     */
    @Override
    public List<AppFamilyVo> selectByMemberId(Long mid) {
        return baseMapper.selectByMemberId(mid);
    }

    @Override
    public List<AppFamilyVo> pageList(LambdaQueryWrapper<TbFamily> wrapper){
        return baseMapper.pageList(wrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public AjaxResult applyFamily(AppMemberVo memberVo){

        TbMemberMiddle middle = middleService.getOne(new LambdaQueryWrapper<TbMemberMiddle>()
                .eq(TbMemberMiddle::getFamilyId,memberVo.getFamilyId())
                .eq(TbMemberMiddle::getMemberId,memberVo.getMemberId())
                .last("limit 1")
        );
        if (middle == null) {
            middle = new TbMemberMiddle();
            middle.setFamilyId(memberVo.getFamilyId());
            middle.setMemberId(memberVo.getMemberId());

            // 新成员名字与会员昵称同步
            memberVo.setName(memberVo.getNickName());
        } else {
            if ("2".equals(middle.getStatus())){
                return AjaxResult.error("已在该家族，不允许重复加入");
            }
        }

        memberVo.setJoins("Y");
        // 保存成员
        if(!userService.saveOrUpdate(memberVo)){
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return AjaxResult.error("申请失败");
        }
        // 是否已加入过家族
        List<TbMemberMiddle> middleList = middleService.list(new LambdaQueryWrapper<TbMemberMiddle>()
                .eq(TbMemberMiddle::getMemberId,memberVo.getMemberId())
                .eq(TbMemberMiddle::getStatus,"2")
        );

        if (middleList.size() == 0) {
            // 如果从未加入过家族，则设置为默认家族
            middle.setAcquiesce("Y");
        }
        middle.setStatus("1"); // 申请
        if(!middleService.saveOrUpdate(middle)){
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return AjaxResult.error("申请失败");
        }
        // 推送
        push(memberVo.getNickName(),memberVo.getFamilyId());
        return AjaxResult.success("申请成功");
    }

    /**
     * 推送
     * @param userName
     * @param familyId
     * @return
     */
    @Async
    protected void push(String userName, Long familyId){
        // 家族信息
        TbFamily family = familyService.getById(familyId);
        if (family == null) {
            return;
        }
        // 管理员列表
        List<AppMemberVo> memberVos = memberService.myMemberList(new LambdaQueryWrapper<AppMemberVo>()
                .eq(AppMemberVo::getFamilyId,familyId)
                //已通过
                .eq(AppMemberVo::getStatus,"2")
                //管理员标志
                .eq(AppMemberVo::getAdmin,"Y")
                //接收消息标志
                .eq(AppMemberVo::getReceive,"Y")
        );
        if (memberVos.size() <= 0) {
            return;
        }
        // 管理员ClientId
        List<String> cids = new ArrayList<>();
        memberVos.forEach(item -> {
            cids.add(item.getClientId());
        });
        if (cids.size() <= 0) {
            return;
        }
        StringBuilder builder = new StringBuilder();
        builder.append(userName);
        builder.append("申请加入");
        builder.append(family.getFullName());
        PushUtil.sendMessage("家族通知",builder.toString(),cids);
    }

    /**
     * 获取家族申请数量
     * @param familyId
     * @return
     */
    @Override
    public int getApplyCount(Long familyId) {
        List<TbMemberMiddle> middles = middleService.list(new LambdaQueryWrapper<TbMemberMiddle>()
                .eq(TbMemberMiddle::getFamilyId,familyId)
                .eq(TbMemberMiddle::getStatus,"1")
        );
        return middles.size();
    }

    @Override
    public boolean saveFamily(TbFamily family){
        if (familyService.save(family)) {
            // TODO 如果是APP端添加，则把添加人设置为家族管理员
            family.setCode(StringUtils.codeFormat(family.getId()));
        }
        return familyService.updateById(family);
    }

    @Override
    public TbFamily myFamily(Long memberId) {
        TbMemberMiddle middle = middleService.getOne(new LambdaQueryWrapper<TbMemberMiddle>()
                .eq(TbMemberMiddle::getMemberId, memberId)
                .eq(TbMemberMiddle::getStatus,"2")
                .eq(TbMemberMiddle::getAcquiesce, "Y")
                .last("limit 1")
        );
        if (middle == null) {
            // 没有默认家族，则设置第一个家族为默认
            middle = middleService.getOne(new LambdaQueryWrapper<TbMemberMiddle>()
                    .eq(TbMemberMiddle::getMemberId, memberId)
                    .eq(TbMemberMiddle::getStatus,"2")
                    .last("limit 1")
            );
            if (middle == null) {
                return null;
            }
            middle.setAcquiesce("Y");
            middleService.saveOrUpdate(middle);
        }
        TbFamily family = familyService.getOne(new LambdaQueryWrapper<TbFamily>()
                .eq(TbFamily::getId, middle.getFamilyId())
                .last("limit 1")
        );
        // 查出家族族长（创建者、群主）
        if (family != null) {
            List<AppMemberVo> memberVos = memberService.selectMemberList(new LambdaQueryWrapper<AppMemberVo>()
                    .eq(AppMemberVo::getFamilyId,family.getId())
                    .eq(AppMemberVo::getPatriarch,"Y")
            );
            if (memberVos.size() > 0) {
                AppMemberVo vo = memberVos.get(0);
                family.setPatriarchName(vo.getName());
            }
        }
        return family;
    }

    @Override
    public boolean change(Long memberId,Long familyId) {
        List<TbMemberMiddle> middleList = middleService.list(new LambdaQueryWrapper<TbMemberMiddle>()
            .eq(TbMemberMiddle::getMemberId, memberId)
        );
        if (middleList.size() <= 0) {
            return false;
        }
        middleList.forEach(item -> {
            item.setAcquiesce("N");
            if (familyId.equals(item.getFamilyId())) {
                item.setAcquiesce("Y");
            }
        });
        return middleService.updateBatchById(middleList);
    }

}
